/* 
 *  Some fundamental physical constants - cgs units
 *
 *  See also: "The Fundamental Physical Constants" by E. Richard
 *     Cohen and Barry N. Taylor (PHYICS TODAY, August 1989).
 * 
 *  A recent discussion:
 *  Harmanec & Prsa, PASP 123, 976 (2011)
 *      http://www.jstor.org/stable/10.1086/661258
 *  Saha (2019) - the new SI units
 *      https://arxiv.org/abs/1911.10204 
 */


#define c_CGS 299792458000.0    /* speed of light [cm/sec] */
#define k_CGS 1.380649e-16      /* Boltzmann  [erg/deg] */
#define h_CGS 6.62607015e-27    /* Planck [erg-sec] */
#define G_CGS 6.67430e-8   	/* gravitional constant 6.6743(2)*/
#define g_MKS 980.665           /* standard acceleration of gravity */

#define Msol_CGS 1.988547e33    /* solar mass in grams */
#define Rsol_CGS 6.95508e10     /* solar radius in cm */
#define Lsol_CGS 3.846e33       /* solar lum in ergs/sec */

#define mH_CGS 1.67e-24		/* mass of hydrogen atom in grams */

#define KPCCM 3.085678e+21	/* kiloparsec in centimeters */
#define GYRSEC 3.155693e+16	/* gigayear in seconds */

#define HI_MHz    1420.40575177 /* 21cm (21.10611405413 cm) line  of HI */
