#
# $Id$
#
.\" -*- nroff -*-
.TH MKISOFS 8 "12 Oct 1997" "Version 1.11.3"
.SH NAME
mkisofs \- create a iso9660 filesystem with optional Rock Ridge attributes.
.SH SYNOPSIS
.B mkisofs
[
.B \-a
]
[
.B \-b
.I boot_image
]
[
.B \-c
.I boot_catalog
]
[
.B \-A
.I application_id
]
[
.B \-f
]
[
.B \-d
]
[
.B \-D
]
[
.B \-i
.I include-list
]
[
.B \-l
]
[
.B \-L
]
[
.B \-L
]
[
.B \-p
.I preparer
]
[
.B \-P
.I publisher
]
[
.B \-r
]
[
.B \-R
]
[
.B \-T
]
[
.B \-v
]
[
.B \-V
]
[
.B \-V
.I volid
]
[
.B \-x
.I path
]
[
.B \-z
]
[
.B \-m
.I glob
]
.B \-o
.I filename
.I path
.SH DESCRIPTION
.B mkisofs
is effectively a pre-mastering program to generate the iso9660 filesystem - it
takes a snapshot of a given directory tree, and generates a binary image which
will correspond to an iso9660 filesystem when written to a block device.
.PP
.B mkisofs
is also capable of generating the System Use Sharing Protocol records specified
by the Rock Ridge Interchange Protocol.  This is used to further describe the
files in the iso9660 filesystem to a unix host, and provides information such
as longer filenames, uid/gid, posix permissions, and block and character
devices.
.PP
Each file written to the iso9660 filesystem must have a filename in the 8.3
format (8 characters, period, 3 characters, all upper case), even if Rock Ridge
is in use.  This filename is used on systems that are not able to make use of
the Rock Ridge extensions (such as MS-DOS), and each filename in each directory
must be different from the other filenames in the same directory.
.B mkisofs
generally tries to form correct names by forcing the unix filename to upper
case and truncating as required, but often times this yields unsatisfactory
results when there are cases where the
truncated names are not all unique.
.B mkisofs
assigns weightings to each filename, and if two names that are otherwise the
same are found the name with the lower priority is renamed to have a 3 digit
number as an extension (where the number is guaranteed to be unique).  An
example of this would be the files foo.bar and
foo.bar.~1~ - the file foo.bar.~1~ would be written as FOO.000;1 and the file
foo.bar would be written as FOO.BAR;1
.PP
Note that
.B mkisofs
is not designed to communicate with the writer directly.  Most writers
have proprietary command sets which vary from one manufacturer to
another, and you need a specialized tool to actually burn the disk.
The
.B cdwrite
utility is one such tool that runs under Linux and performs this task.
The latest version of
.B cdwrite
is capable of communicating with Phillips/IMS/Kodak, HP and Yamaha drives.
Most writers come with some version of DOS software that allows a direct image
copy of an iso9660 image to the writer.  The current version of
.B cdwrite
is available from sunsite.unc.edu: /utils/disk-management/cdwrite-2.0.tar.gz
.PP
Also you should know that most cd writers are very particular about timing.
Once you start to burn a disc, you cannot let their buffer empty before you
are done, or you will end up with a corrupt disc.  Thus it is critical
that you be able to maintain an uninterrupted data stream to the writer
for the entire time that the disc is being written.
.PP
.br
.B path
is the path of the directory tree to be copied into the iso9660 filesystem.
.SH OPTIONS
.TP
.I \-a
Include all files on the iso9660 filesystem.  Normally files that contain the
characters '~' or '#' will not be included (these are typically backup files
for editors under unix).
.TP
.I \-A application_id
Specifies a text string that will be written into the volume header.
This should describe the application that will be on the disc.  There
is space on the disc for 128 characters of information.  This parameter can
also be set in the file
.I \&.mkisofsrc
with APPI=id.
If specified in both places, the command line version is used.
.TP
.I \-b boot_image
Specifies the path and filename of the boot image to be used when making
an "El Torito" bootable CD. The pathname must be relative to the source
path specified to
.B mkisofs.
This option is required to make a bootable CD.
The boot image must be exactly the size of either a 1.2, 1.44, or a 2.88
meg floppy, and
.B mkisofs
will use this size when creating the output iso9660
filesystem. It is assumed that the first 512 byte sector should be read
from the boot image (it is essentially emulating a normal floppy drive).
This will work, for example, if the boot image is a LILO based boot floppy.
.TP
.I \-c boot_catalog
Specifies the path and filename of the boot catalog to be used when making
an "El Torito" bootable CD. The pathname must be relative to the source
path specified to
.B mkisofs.
This option is required to make a bootable CD.
This file will be created by
.B mkisofs
in the source filesystem, so be
sure the specified filename does not conflict with an existing file, as
it will be quietly overwritten! Usually a name like "boot.catalog" is
chosen.
.TP
.I \-d
Omit trailing period from files that do not have a period.  This violates the
ISO9660 standard, but it happens to work on many systems.  Use with caution.
.TP
.I \-D
Do not use deep directory relocation, and instead just pack them in the
way we see them.  This violates the ISO9660 standard, but it works on many
systems.  Use with caution.
.TP
.I \-f
Follow symbolic links when generating the filesystem.  When this option is not
in use, symbolic links will be entered using Rock Ridge if enabled, otherwise
the file will be ignored.
.TP
.I \-i include-list
Use the specified file as a list of files to add to the directory tree.
This is useful for quickly repacking a CD while adding files to it.
The format of this file is path1/file=path2 where path1 is the directory
in the ISO9660 file system where file should appear and path2 is the
where to find the file.  NOTE: This option doesn't work well, and
needs to be compltely redone so that integration with the rest of mkisofs
is handled in a cleaner fashion.
.TP
.I \-l
Allow full 32 character filenames.  Normally the ISO9660 filename will be in an
8.3 format which is compatible with MS-DOS, even though the ISO9660 standard
allows filenames of up to 32 characters.  If you use this option, the disc may
be difficult to use on a MS-DOS system, but this comes in handy on some other
systems (such as the Amiga).  Use with caution.
.TP
.I \-L
Allow filenames to begin with a period.  Usually, a leading dot is
replaced with an underscore in order to maintain MS-DOS compatibility.
.TP
.I \-m glob
Exclude
.I glob
from being written to CDROM.
.I glob
is a shell wild-card-style pattern that must match part of the filename (not 
the path as with option
.BR -x ).
Technically
.I glob
is matched against the
.I d->d_name
part of the directory entry.
Multiple globs may be excluded (up to 1000).
Example:

mkisofs \-o rom \-m '*.o' \-m core \-m foobar

would exclude all files ending in ".o", called "core" or "foobar" to be
copied to CDROM. Note that if you had a directory called "foobar" it too (and
of course all its descendants) would be excluded.
.TP
.I \-M path
Specifies path to existing iso9660 image to be merged.  The output
of 
.B mkisofs
will be a new session which should get written to the end of the
image specified in -M.  Typically this requires multi-session capability
for the recorder and cdrom drive that you are attempting to write this
image to.  Support for this is not yet 100% complete, because some handshaking
is required between mkisofs and cdwrite in order to determine the next
writable address on the cdrom.
.TP
.I \-N
Omit version numbers from ISO9660 file names.  This may violate the ISO9660
standard, but no one really uses the version numbers anyway.  Use with caution.
.TP
.I \-o filename
is the name of the file to which the iso9660 filesystem image should be
written.  This can be a disk file, a tape drive, or it can correspond directly
to the device name of the optical disc writer.  If not specified, stdout is
used.  Note that the output can also be a block special device for a regular
disk drive, in which case the disk partition can be mounted and examined to
ensure that the premastering was done correctly.
.TP
.I \-P publisher_id
Specifies a text string that will be written into the volume header.
This should describe the publisher of the CDROM, usually with a
mailing address and phone number.  There is space on the disc for 128
characters of information.  This parameter can also be set in the file
.I \&.mkisofsrc
with PUBL=.
If specified in both places, the command line version is used.
.TP
.I \-p preparer_id
Specifies a text string that will be written into the volume header.
This should describe the preparer of the CDROM, usually with a mailing
address and phone number.  There is space on the disc for 128
characters of information.  This parameter can also be set in the file
.I \&.mkisofsrc
with PREP=.
If specified in both places, the command line version is used.
.TP
.I \-R
Generate SUSP and RR records using the Rock Ridge protocol to further describe
the files on the iso9660 filesystem.
.TP
.I \-r
This is like the \-R option, but file ownership and modes are set to
more useful values.  The uid and gid are set to zero, because they are
usually only useful on the author's system, and not useful to the
client.  All the file read bits are set true, so that files and
directories are globally readable on the client.  If any execute bit is
set for a file, set all of the execute bits, so that executables are
globally executable on the client.  If any search bit is set for a
directory, set all of the search bits, so that directories are globally
searchable on the client.  All write bits are cleared, because the
CD-Rom will be mounted read-only in any case.  If any of the special
mode bits are set, clear them, because file locks are not useful on a
read-only file system, and set-id bits are not desirable for uid 0 or
gid 0.
.TP
.I \-T
Generate a file TRANS.TBL in each directory on the CDROM, which can be used
on non-Rock Ridge capable systems to help establish the correct file names.
There is also information present in the file that indicates the major and
minor numbers for block and character devices, and each symlink has the name of
the link file given.
.TP
.I \-V volid
Specifies the volume ID to be written into the master block.  This
parameter can also be set in the file
.I \&.mkisofsrc
with VOLI=id.
If specified in both places, the command line version is used.
.TP
.I \-v
Verbose execution.
.TP
.I \-x path
Exclude
.I path
from being written to CDROM.
.I path
must be the complete pathname that results from concatenating the pathname
given as command line argument and the path relative to this directory.
Multiple paths may be excluded (up to 1000).
Example: 

mkisofs \-o cd \-x /local/dir1 \-x /local/dir2 /local
.TP
.I \-z
Generate special SUSP records for transparently compressed files.  This is
only of use and interest for hosts that support transparent decompression.
This is an experimental feature, and no hosts yet support this, but there
are ALPHA patches for Linux that can make use of this feature.
.SH CONFIGURATION
.B mkisofs
looks for the
.IR \&.mkisofsrc
file,
first in the current working directory,
then in the user's home directory,
and then in the directory in which the
.B mkisofs
binary is stored.  This file is assumed to contain a series of lines
of the form "TAG=value", and in this way you can specify certain
options.
The case of the tag is not significant.
Some fields in the volume header
are not settable on the command line, but can be altered through this
facility.
Comments may be placed in this file,
using lines which start with a hash (#) character.
.TP
APPI
The application identifier
should describe the application that will be on the disc.
There is space on the disc for 128 characters of information.
May be overridden using the \-A command line option.
.TP
COPY
The copyright information,
often the name of a file on the disc containing the copyright notice.
There is space in the disc for 37 characters of information.
.TP
ABST
The abstract information,
often the name of a file on the disc containing an abstract.
There is space in the disc for 37 characters of information.
.TP
BIBL
The bibliographic information,
often the name of a file on the disc containing a bibliography.
There is space in the disc for 37 characters of information.
.TP
PREP
This should describe the preparer of the CDROM,
usually with a mailing address and phone number.
There is space on the disc for 128 characters of information.
May be overridden using the \-p command line option.
.TP
PUBL
This should describe the publisher of the CDROM,
usually with a mailing address and phone number.
There is space on the disc for 128 characters of information.
May be overridden using the \-P command line option.
.TP
SYSI
The System Identifier.
There is space on the disc for 32 characters of information.
.TP
VOLI
The Volume Identifier.
There is space on the disc for 32 characters of information.
May be overridden using the \-V command line option.
.TP
VOLS
The Volume Set Name.
There is space on the disc for 278 characters of information.
.PP
.B mkisofs
can also be configured at compile time with defaults for many of these fields.
See the file defaults.h.
.SH AUTHOR
.B mkisofs
is not based on the standard mk*fs tools for unix, because we must generate
a complete  copy of an existing filesystem on a disk in the  iso9660
filesystem.  The name mkisofs is probably a bit of a misnomer, since it
not only creates the filesystem, but it also populates it as well.
.PP
.br
Eric Youngdale <ericy@gnu.ai.mit.edu> or <eric@andante.jic.com> wrote both the
Linux isofs9660 filesystem and the mkisofs utility, and is currently
maintaining them.  The copyright for the mkisofs utility is held by
Yggdrasil Computing, Incorporated.
.SH BUGS
Any files that have hard links to files not in the tree being copied to the
iso9660 filessytem will have an incorrect file reference count.
.PP
There may be some other ones.  Please, report them to the author.
.SH FUTURE IMPROVEMENTS
Allow specification of multiple paths on the command line to be included in
iso9660 filesystem.  Can be tricky - directory entries in the root directory
need to be properly sorted.
.SH AVAILABILITY
.B mkisofs
is available for anonymous ftp from tsx-11.mit.edu in
/pub/linux/packages/mkisofs and many other mirror sites.
